package cursor_auth

import (
	"bytes"
	"database/sql"
	"encoding/json"
	"fmt"
	"io"
	"io/ioutil"
	"net"
	"net/http"
	"os"
	"path/filepath"
	"runtime"
	"strings"
	"time"

	_ "github.com/mattn/go-sqlite3"
)

// CheckCard 检查卡片状态
func CheckCard(apiBaseURL, cardNumber string) error {
	resp, err := http.Get(fmt.Sprintf("%s/cards/check/%s", apiBaseURL, cardNumber))
	if err != nil {
		return fmt.Errorf("检查卡片失败: %v", err)
	}
	defer resp.Body.Close()

	if resp.StatusCode == http.StatusNotFound {
		return fmt.Errorf("无效的激活码")
	}

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("检查卡片失败: HTTP %d", resp.StatusCode)
	}

	var result struct {
		IsUsed bool `json:"is_used"`
		Days   int  `json:"days"`
	}

	if err := json.NewDecoder(resp.Body).Decode(&result); err != nil {
		return fmt.Errorf("解析响应失败: %v", err)
	}

	if result.IsUsed {
		return fmt.Errorf("该激活码已被使用")
	}

	return nil
}

// CursorAuthManager 管理Cursor的认证信息
type CursorAuthManager struct {
	DBPath string
}

// AuthResponse API响应结构
type AuthResponse struct {
	AccessToken  string `json:"access_token"`
	RefreshToken string `json:"refresh_token"`
	Email        string `json:"email"`
}

// NewAuthManager 创建一个新的认证管理器
func NewAuthManager() *CursorAuthManager {
	var cursorDir string
	switch runtime.GOOS {
	case "windows":
		appData := os.Getenv("APPDATA")
		if appData == "" {
			return nil
		}
		cursorDir = filepath.Join(appData, "Cursor", "User", "globalStorage")
	case "darwin":
		home, err := os.UserHomeDir()
		if err != nil {
			return nil
		}
		cursorDir = filepath.Join(home, "Library", "Application Support", "Cursor", "User", "globalStorage")
	case "linux":
		home, err := os.UserHomeDir()
		if err != nil {
			return nil
		}
		cursorDir = filepath.Join(home, ".config", "Cursor", "User", "globalStorage")
	default:
		return nil
	}

	if err := os.MkdirAll(cursorDir, 0755); err != nil {
		return nil
	}

	manager := &CursorAuthManager{
		DBPath: filepath.Join(cursorDir, "state.vscdb"),
	}

	if err := manager.initDatabase(); err != nil {
		return nil
	}

	return manager
}

// initDatabase 初始化数据库
func (m *CursorAuthManager) initDatabase() error {
	db, err := sql.Open("sqlite3", m.DBPath)
	if err != nil {
		return fmt.Errorf("打开数据库失败: %v", err)
	}
	defer db.Close()

	_, err = db.Exec(`
		CREATE TABLE IF NOT EXISTS itemTable (
			key TEXT PRIMARY KEY,
			value TEXT
		)
	`)
	if err != nil {
		return fmt.Errorf("创建表失败: %v", err)
	}

	return nil
}

// UpdateAuth 更新认证信息
func (m *CursorAuthManager) UpdateAuth(email, accessToken, refreshToken string) error {
	db, err := sql.Open("sqlite3", m.DBPath)
	if err != nil {
		return err
	}
	defer db.Close()

	if err := db.Ping(); err != nil {
		return err
	}

	updates := []struct {
		key   string
		value string
	}{
		{"cursorAuth/cachedSignUpType", "Auth_0"},
		{"cursorAuth/signUpType", "Auth_0"},
		{"cursorAuth/isLoggedIn", "true"},
		{"cursorAuth/stripeMembershipType", "free_trial"},
		{"cursorAuth/loginType", "Auth_0"},
		{"cursorAuth/cachedLoginType", "Auth_0"},
		{"cursorAuth/membershipType", "free_trial"},
		{"cursorAuth/cachedMembershipType", "free_trial"},
		{"cursorAuth/isAuthenticated", "true"},
		{"cursorAuth/isAuthorized", "true"},
		{"cursorAuth/isActivated", "true"},
		{"cursorAuth/status", "active"},
	}

	if email != "" {
		updates = append(updates,
			struct{ key, value string }{"cursorAuth/cachedEmail", email},
			struct{ key, value string }{"cursorAuth/email", email},
			struct{ key, value string }{"cursorAuth/username", email},
			struct{ key, value string }{"cursorAuth/cachedUsername", email},
		)
	}

	if refreshToken != "" && strings.HasPrefix(refreshToken, "ey") {
		updates = append(updates,
			struct{ key, value string }{"cursorAuth/cachedAccessToken", refreshToken},
			struct{ key, value string }{"cursorAuth/accessToken", refreshToken},
			struct{ key, value string }{"cursorAuth/token", refreshToken},
			struct{ key, value string }{"cursorAuth/refreshToken", refreshToken},
			struct{ key, value string }{"cursorAuth/cachedRefreshToken", refreshToken},
			struct{ key, value string }{"cursorAuth/idToken", refreshToken},
			struct{ key, value string }{"cursorAuth/cachedIdToken", refreshToken},
		)
	}

	tx, err := db.Begin()
	if err != nil {
		return err
	}

	_, err = tx.Exec(`
		CREATE TABLE IF NOT EXISTS itemTable (
			key TEXT PRIMARY KEY,
			value TEXT
		)
	`)
	if err != nil {
		tx.Rollback()
		return err
	}

	for _, update := range updates {
		_, err := tx.Exec(`
			INSERT OR REPLACE INTO itemTable (key, value)
			VALUES (?, ?)
		`, update.key, update.value)
		if err != nil {
			tx.Rollback()
			return err
		}
	}

	if err := tx.Commit(); err != nil {
		return err
	}

	return nil
}

// GetAuthInfo 获取认证信息
func (m *CursorAuthManager) GetAuthInfo() (map[string]string, error) {
	db, err := sql.Open("sqlite3", m.DBPath)
	if err != nil {
		return nil, err
	}
	defer db.Close()

	rows, err := db.Query("SELECT key, value FROM itemTable WHERE key LIKE 'cursorAuth/%'")
	if err != nil {
		return nil, err
	}
	defer rows.Close()

	info := make(map[string]string)
	for rows.Next() {
		var key, value string
		if err := rows.Scan(&key, &value); err != nil {
			return nil, err
		}
		info[key] = value
	}

	return info, nil
}

// ClearAuthInfo 清理认证信息
func (m *CursorAuthManager) ClearAuthInfo() error {
	home, err := os.UserHomeDir()
	if err != nil {
		return err
	}

	tokenPath := filepath.Join(home, ".cursor_auth", "token")
	if _, err := os.Stat(tokenPath); err == nil {
		if err := os.Remove(tokenPath); err != nil {
			return err
		}
	}

	db, err := sql.Open("sqlite3", m.DBPath)
	if err != nil {
		return err
	}
	defer db.Close()

	_, err = db.Exec("DELETE FROM itemTable WHERE key LIKE 'cursorAuth/%'")
	return err
}

// ActivateCard 激活卡片
func ActivateCard(apiBaseURL, cardNumber string) (*AuthResponse, error) {
	if err := CheckCard(apiBaseURL, cardNumber); err != nil {
		return nil, err
	}

	payload := map[string]string{"card_number": cardNumber}
	jsonData, err := json.Marshal(payload)
	if err != nil {
		return nil, err
	}

	resp, err := http.Post(
		fmt.Sprintf("%s/auth/activate-card", apiBaseURL),
		"application/json",
		strings.NewReader(string(jsonData)),
	)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()

	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return nil, err
	}
	if resp.StatusCode != http.StatusOK {
		var errorResponse struct {
			Detail string `json:"detail"`
		}
		if err := json.Unmarshal(body, &errorResponse); err == nil && errorResponse.Detail != "" {
			return nil, fmt.Errorf("激活失败: %s", errorResponse.Detail)
		}
		return nil, fmt.Errorf("激活失败: HTTP %d", resp.StatusCode)
	}

	var result AuthResponse
	if err := json.Unmarshal(body, &result); err != nil {
		return nil, err
	}

	if result.AccessToken == "" {
		return nil, fmt.Errorf("激活响应缺少必要的token信息")
	}

	if err := saveToken(result.AccessToken); err != nil {
		return nil, fmt.Errorf("保存Token失败: %v", err)
	}

	if result.RefreshToken == "" {
		result.RefreshToken = result.AccessToken
	}

	if result.Email == "" {
		result.Email = fmt.Sprintf("%s@temp.com", cardNumber)
	}

	return &result, nil
}

// GetToken 获取认证token
func GetToken() (string, error) {
	home, err := os.UserHomeDir()
	if err != nil {
		return "", err
	}

	tokenPath := filepath.Join(home, ".cursor_auth", "token")
	data, err := ioutil.ReadFile(tokenPath)
	if err != nil {
		if os.IsNotExist(err) {
			return "", nil
		}
		return "", err
	}

	return strings.TrimSpace(string(data)), nil
}

// SaveToken 保存认证token
func saveToken(token string) error {
	home, err := os.UserHomeDir()
	if err != nil {
		return err
	}

	tokenDir := filepath.Join(home, ".cursor_auth")
	if err := os.MkdirAll(tokenDir, 0755); err != nil {
		return err
	}

	tokenPath := filepath.Join(tokenDir, "token")
	return ioutil.WriteFile(tokenPath, []byte(token), 0644)
}

// UpdateCursorAuth 更新Cursor的认证信息
func (m *CursorAuthManager) UpdateCursorAuth() error {
	settings := NewSettings()
	if settings == nil {
		return fmt.Errorf("创建settings失败")
	}

	token := settings.GetToken()
	if token == "" {
		return fmt.Errorf("未找到token")
	}

	requestData := map[string]interface{}{
		"client_version": "1.0.0",
		"system":         runtime.GOOS,
		"timestamp":      time.Now().Unix(),
	}

	jsonData, err := json.Marshal(requestData)
	if err != nil {
		return fmt.Errorf("构造请求数据失败: %v", err)
	}

	apiBaseURL := os.Getenv("API_BASE_URL")
	if apiBaseURL == "" {
		apiBaseURL = "https://cursor-s.gzdora.cn/api"
	}

	req, err := http.NewRequest("POST", fmt.Sprintf("%s/auth/get-cursor-token", apiBaseURL), bytes.NewBuffer(jsonData))
	if err != nil {
		return fmt.Errorf("创建请求失败: %v", err)
	}

	req.Header.Set("Authorization", fmt.Sprintf("Bearer %s", token))
	req.Header.Set("Content-Type", "application/json")

	client := &http.Client{Timeout: 30 * time.Second}
	resp, err := client.Do(req)
	if err != nil {
		if err, ok := err.(net.Error); ok && err.Timeout() {
			return fmt.Errorf("请求超时，请重新点击")
		}
		return fmt.Errorf("发送请求失败: %v", err)
	}
	defer resp.Body.Close()

	body, _ := io.ReadAll(resp.Body)
	if resp.StatusCode != http.StatusOK {
		var errorResponse struct {
			Detail string `json:"detail"`
		}
		if err := json.Unmarshal(body, &errorResponse); err == nil && errorResponse.Detail != "" {
			// 不清除本地token，让用户可以重试
			return fmt.Errorf("%s", errorResponse.Detail)
		}
		return fmt.Errorf("获取Cursor token失败: %s", string(body))
	}

	var result struct {
		ID           int                    `json:"id"`
		Email        string                 `json:"email"`
		AccessToken  string                 `json:"access_token"`
		RefreshToken string                 `json:"refresh_token"`
		Status       string                 `json:"status"`
		PullCount    int                    `json:"pull_count"`
		Usage        map[string]interface{} `json:"usage"`
	}

	if err := json.Unmarshal(body, &result); err != nil {
		return fmt.Errorf("解析响应失败: %v", err)
	}

	// 检查响应中的token是否有效
	if result.RefreshToken == "" || !strings.HasPrefix(result.RefreshToken, "ey") {
		return fmt.Errorf("获取到的token无效")
	}

	db, err := sql.Open("sqlite3", m.DBPath)
	if err != nil {
		return fmt.Errorf("打开数据库失败: %v", err)
	}
	defer db.Close()

	if err := db.Ping(); err != nil {
		return fmt.Errorf("数据库连接测试失败: %v", err)
	}

	tx, err := db.Begin()
	if err != nil {
		return fmt.Errorf("开始事务失败: %v", err)
	}

	_, err = tx.Exec(`
		CREATE TABLE IF NOT EXISTS itemTable (
			key TEXT PRIMARY KEY,
			value TEXT
		)
	`)
	if err != nil {
		tx.Rollback()
		return fmt.Errorf("创建表失败: %v", err)
	}

	updates := []struct {
		key   string
		value string
	}{
		{"cursorAuth/cachedSignUpType", "Auth_0"},
		{"cursorAuth/signUpType", "Auth_0"},
		{"cursorAuth/isLoggedIn", "true"},
		{"cursorAuth/stripeMembershipType", "free_trial"},
		{"cursorAuth/loginType", "Auth_0"},
		{"cursorAuth/cachedLoginType", "Auth_0"},
		{"cursorAuth/membershipType", "free_trial"},
		{"cursorAuth/cachedMembershipType", "free_trial"},
		{"cursorAuth/isAuthenticated", "true"},
		{"cursorAuth/isAuthorized", "true"},
		{"cursorAuth/isActivated", "true"},
		{"cursorAuth/status", "active"},
	}

	if result.Email != "" {
		updates = append(updates,
			struct{ key, value string }{"cursorAuth/cachedEmail", result.Email},
			struct{ key, value string }{"cursorAuth/email", result.Email},
			struct{ key, value string }{"cursorAuth/username", result.Email},
			struct{ key, value string }{"cursorAuth/cachedUsername", result.Email},
		)
	}

	if result.RefreshToken != "" && strings.HasPrefix(result.RefreshToken, "ey") {
		updates = append(updates,
			struct{ key, value string }{"cursorAuth/cachedAccessToken", result.RefreshToken},
			struct{ key, value string }{"cursorAuth/accessToken", result.RefreshToken},
			struct{ key, value string }{"cursorAuth/token", result.RefreshToken},
			struct{ key, value string }{"cursorAuth/refreshToken", result.RefreshToken},
			struct{ key, value string }{"cursorAuth/cachedRefreshToken", result.RefreshToken},
			struct{ key, value string }{"cursorAuth/idToken", result.RefreshToken},
			struct{ key, value string }{"cursorAuth/cachedIdToken", result.RefreshToken},
		)
	}

	for _, update := range updates {
		_, err := tx.Exec(`
			INSERT OR REPLACE INTO itemTable (key, value)
			VALUES (?, ?)
		`, update.key, update.value)
		if err != nil {
			tx.Rollback()
			return fmt.Errorf("更新键值对失败 [%s]: %v", update.key, err)
		}
	}

	if err := tx.Commit(); err != nil {
		return fmt.Errorf("提交事务失败: %v", err)
	}

	return nil
}
