package cursor_auth

import (
	"encoding/json"
	"fmt"
	"os"
	"path/filepath"
)

// Settings 管理应用程序设置
type Settings struct {
	configDir string
}

// NewSettings 创建一个新的设置管理器
func NewSettings() *Settings {
	homeDir, err := os.UserHomeDir()
	if err != nil {
		return &Settings{configDir: "."}
	}

	configDir := filepath.Join(homeDir, ".cursor_auth")
	if err := os.MkdirAll(configDir, 0755); err != nil {
		return &Settings{configDir: "."}
	}

	return &Settings{configDir: configDir}
}

// load 从文件加载设置
func (s *Settings) load() (map[string]string, error) {
	configFile := filepath.Join(s.configDir, "config.json")
	if _, err := os.Stat(configFile); os.IsNotExist(err) {
		return make(map[string]string), nil
	}

	data, err := os.ReadFile(configFile)
	if err != nil {
		return nil, fmt.Errorf("读取配置文件失败: %v", err)
	}

	var config map[string]string
	if err := json.Unmarshal(data, &config); err != nil {
		return nil, fmt.Errorf("解析配置文件失败: %v", err)
	}

	return config, nil
}

// save 保存设置到文件
func (s *Settings) save(config map[string]string) error {
	data, err := json.Marshal(config)
	if err != nil {
		return fmt.Errorf("序列化配置失败: %v", err)
	}

	configFile := filepath.Join(s.configDir, "config.json")
	if err := os.WriteFile(configFile, data, 0644); err != nil {
		return fmt.Errorf("保存配置文件失败: %v", err)
	}

	return nil
}

// SaveCardNumber 保存卡片号码
func (s *Settings) SaveCardNumber(cardNumber string) {
	config, _ := s.load()
	config["card_number"] = cardNumber
	s.save(config)
}

// GetCardNumber 获取保存的卡片号码
func (s *Settings) GetCardNumber() string {
	config, _ := s.load()
	return config["card_number"]
}

// SaveToken 保存token
func (s *Settings) SaveToken(token string) {
	config, _ := s.load()
	config["token"] = token
	s.save(config)
}

// GetToken 获取保存的token
func (s *Settings) GetToken() string {
	config, _ := s.load()
	return config["token"]
}
