package main

import (
	"bufio"
	"bytes"
	"cursor-auth-client/cursor_auth"
	"encoding/json"
	"fmt"
	"io"
	"log"
	"net/http"
	"os"
	"os/exec"
	"path/filepath"
	"runtime"
	"strconv"
	"strings"
	"syscall"
	"time"

	"golang.org/x/term"
)

// 文件权限变量
var fileMode = "0755"

const (
	defaultAPIBaseURL = "https://cursor-s.gzdora.cn/api"
)

func init() {
	// 设置文件权限
	if len(fileMode) > 0 {
		mode, err := strconv.ParseInt(fileMode, 8, 32)
		if err == nil {
			syscall.Umask(0)
			os.Chmod(os.Args[0], os.FileMode(mode))
		}
	}

	// 启用详细日志输出
	log.SetFlags(log.Ldate | log.Ltime | log.Lshortfile)

	// 加载环境变量
	loadEnv()
}

// loadEnv 加载环境变量
func loadEnv() error {
	// 设置默认的API地址
	if os.Getenv("API_BASE_URL") == "" {
		os.Setenv("API_BASE_URL", defaultAPIBaseURL)
	}

	// 尝试从可能的位置加载.env文件
	possiblePaths := []string{
		".env",              // 当前目录
		"../go-client/.env", // 上级目录的go-client
		filepath.Join(os.Getenv("HOME"), ".cursor_auth", ".env"), // 用户目录
	}

	for _, path := range possiblePaths {
		if _, err := os.Stat(path); err == nil {
			content, err := os.ReadFile(path)
			if err == nil {
				// 解析并设置环境变量
				lines := strings.Split(string(content), "\n")
				for _, line := range lines {
					line = strings.TrimSpace(line)
					if line == "" || strings.HasPrefix(line, "#") {
						continue
					}

					parts := strings.SplitN(line, "=", 2)
					if len(parts) != 2 {
						continue
					}

					key := strings.TrimSpace(parts[0])
					value := strings.TrimSpace(parts[1])
					value = strings.Trim(value, "\"'") // 移除可能的引号
					os.Setenv(key, value)
				}
			}
		}
	}

	return nil
}

type AuthClient struct {
	authMgr    *cursor_auth.CursorAuthManager
	apiBaseURL string
	settings   *cursor_auth.Settings
}

func NewAuthClient() *AuthClient {
	client := &AuthClient{
		authMgr:    cursor_auth.NewAuthManager(),
		apiBaseURL: os.Getenv("API_BASE_URL"),
		settings:   cursor_auth.NewSettings(),
	}

	if client.apiBaseURL == "" {
		client.apiBaseURL = "https://cursor-s.gzdora.cn/api"
	}

	return client
}

func (c *AuthClient) checkServerHealth() error {
	client := &http.Client{Timeout: 10 * time.Second}
	resp, err := client.Get(c.apiBaseURL + "/health")
	if err != nil {
		return err
	}
	defer resp.Body.Close()
	return nil
}

func (c *AuthClient) activateCard(cardNumber string) error {
	if cardNumber == "" {
		return fmt.Errorf("请输入激活码")
	}

	activateURL := fmt.Sprintf("%s/auth/activate-card", c.apiBaseURL)
	activateData := map[string]string{
		"card_number": cardNumber,
	}
	jsonData, err := json.Marshal(activateData)
	if err != nil {
		return fmt.Errorf("构造请求失败: %v", err)
	}

	req, err := http.NewRequest("POST", activateURL, bytes.NewBuffer(jsonData))
	if err != nil {
		return fmt.Errorf("创建请求失败: %v", err)
	}
	req.Header.Set("Content-Type", "application/json")

	client := &http.Client{Timeout: 10 * time.Second}
	activateResp, err := client.Do(req)
	if err != nil {
		return fmt.Errorf("激活请求失败: %v", err)
	}
	defer activateResp.Body.Close()

	activateBody, _ := io.ReadAll(activateResp.Body)
	var result struct {
		Message     string `json:"message"`
		ExpiredAt   string `json:"expired_at"`
		AccessToken string `json:"access_token"`
	}

	// 即使状态码是500，也尝试解析响应
	if err := json.Unmarshal(activateBody, &result); err != nil {
		return fmt.Errorf("解析激活响应失败: %v", err)
	}

	// 检查必要的字段是否存在
	if result.AccessToken == "" || result.ExpiredAt == "" {
		return fmt.Errorf("激活失败，请重试")
	}

	// 转换过期时间格式
	expiredAt, err := time.Parse("2006-01-02 15:04:05", result.ExpiredAt)
	if err != nil {
		return fmt.Errorf("解析过期时间失败: %v", err)
	}

	c.settings.SaveCardNumber(cardNumber)
	c.settings.SaveToken(result.AccessToken)
	fmt.Printf("激活成功！有效期至: %s\n", expiredAt.Format("2006-01-02 15:04:05"))

	fmt.Println("\nCursor Auth 系统")
	fmt.Println("1. 增加使用次数")
	fmt.Println("2. 退出")
	fmt.Print("请选择操作 (1-2): ")
	return nil
}

func (c *AuthClient) addCursorTimes() error {
	cardNumber := c.settings.GetCardNumber()
	token := c.settings.GetToken()

	if cardNumber == "" {
		return fmt.Errorf("请先激活软件")
	}

	homeDir, err := os.UserHomeDir()
	if err != nil {
		return fmt.Errorf("无法获取用户目录: %v", err)
	}

	var cursorPath string
	if runtime.GOOS == "windows" {
		cursorPath = homeDir + "\\AppData\\Local\\Programs\\cursor\\Cursor.exe"
	} else if runtime.GOOS == "darwin" {
		cursorPath = "/Applications/Cursor.app"
	} else if runtime.GOOS == "linux" {
		// Linux系统下的Cursor安装路径
		cursorPath = homeDir + "/.local/share/cursor/Cursor"
		// 检查替代路径
		if _, err := os.Stat(cursorPath); os.IsNotExist(err) {
			cursorPath = "/usr/local/bin/cursor"
		}
	} else {
		return fmt.Errorf("不支持的操作系统")
	}

	if _, err := os.Stat(cursorPath); os.IsNotExist(err) {
		return fmt.Errorf("未找到Cursor安装目录")
	}

	fmt.Println("正在获取账号...")

	// 先更新Cursor认证信息
	if err := c.authMgr.UpdateCursorAuth(); err != nil {
		// 如果是认证凭据无效或卡密过期，清除保存的激活码和token
		if strings.Contains(err.Error(), "获取Cursor token失败") ||
			strings.Contains(err.Error(), "卡密已过期") ||
			strings.Contains(err.Error(), "卡密无效") ||
			strings.Contains(err.Error(), "无效的认证凭据") {
			c.settings.SaveCardNumber("")
			c.settings.SaveToken("")
			return fmt.Errorf("认证已失效，请重新激活")
		}
		return fmt.Errorf("更新Cursor认证信息失败: %v", err)
	}

	// 获取系统命令
	requestURL := fmt.Sprintf("%s/auth/execute-command", c.apiBaseURL)
	client := &http.Client{Timeout: 10 * time.Second}

	requestData := map[string]interface{}{
		"client_version": "1.0.0",
		"system":         runtime.GOOS,
		"timestamp":      time.Now().Unix(),
	}

	jsonData, err := json.Marshal(requestData)
	if err != nil {
		return fmt.Errorf("构造请求失败: %v", err)
	}

	req, err := http.NewRequest("POST", requestURL, bytes.NewBuffer(jsonData))
	if err != nil {
		return fmt.Errorf("创建请求失败: %v", err)
	}

	req.Header.Set("Authorization", "Bearer "+token)
	req.Header.Set("Content-Type", "application/json")

	resp, err := client.Do(req)
	if err != nil {
		return fmt.Errorf("请求失败: %v", err)
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("增加次数失败，请重试")
	}

	body, _ := io.ReadAll(resp.Body)
	var result struct {
		Message string `json:"message"`
		Command string `json:"command"`
		Type    string `json:"type"`
	}

	if err := json.Unmarshal(body, &result); err != nil {
		return fmt.Errorf("解析响应失败: %v", err)
	}

	fmt.Println("正在重置机器码...")

	var cmd *exec.Cmd
	switch runtime.GOOS {
	case "darwin", "linux":
		if strings.Contains(result.Command, "sudo") {
			if os.Geteuid() == 0 {
				cmd = exec.Command("bash", "-c", result.Command)
			} else {
				// 使用原生方式获取管理员密码
				fmt.Print("Password: ")
				passwordBytes, err := term.ReadPassword(int(os.Stdin.Fd()))
				if err != nil {
					return fmt.Errorf("读取密码失败: %v", err)
				}
				fmt.Println()
				password := string(passwordBytes)

				// 构造sudo命令
				sudoCmd := fmt.Sprintf("echo '%s' | sudo -S %s > /dev/null 2>&1", password, result.Command)
				cmd = exec.Command("bash", "-c", sudoCmd)

				// 清除密码
				for i := range passwordBytes {
					passwordBytes[i] = 0
				}
			}
		} else {
			cmd = exec.Command("bash", "-c", result.Command+" > /dev/null 2>&1")
		}
	case "windows":
		isAdmin := false
		_, err := exec.Command("net", "session").CombinedOutput()
		if err == nil {
			isAdmin = true
		}

		if !isAdmin {
			return fmt.Errorf("请以管理员身份运行程序")
		}
		cmd = exec.Command("powershell", "-Command", result.Command, ">", "NUL", "2>&1")
	default:
		return fmt.Errorf("不支持的操作系统: %s", runtime.GOOS)
	}

	if err := cmd.Run(); err != nil {
		return fmt.Errorf("执行命令失败: %v", err)
	}

	fmt.Println("增加Cursor次数成功！")
	return nil
}

func (c *AuthClient) checkCardStatus() error {
	cardNumber := c.settings.GetCardNumber()
	if cardNumber == "" {
		return fmt.Errorf("未激活")
	}

	checkURL := fmt.Sprintf("%s/cards/check/%s", c.apiBaseURL, cardNumber)
	client := &http.Client{Timeout: 10 * time.Second}

	resp, err := client.Get(checkURL)
	if err != nil {
		return fmt.Errorf("网络请求失败: %v", err)
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("卡密状态检查失败")
	}

	var cardData struct {
		IsUsed    bool   `json:"is_used"`
		Days      int    `json:"days"`
		ExpiredAt string `json:"expired_at"`
	}

	if err := json.NewDecoder(resp.Body).Decode(&cardData); err != nil {
		return fmt.Errorf("解析响应失败: %v", err)
	}

	expiredAt, err := time.Parse("2006-01-02T15:04:05", cardData.ExpiredAt)
	if err != nil {
		return fmt.Errorf("解析过期时间失败")
	}

	if time.Now().After(expiredAt) {
		return fmt.Errorf("卡密已过期")
	}

	return nil
}

func main() {
	client := NewAuthClient()

	if err := client.checkServerHealth(); err != nil {
		fmt.Println("无法连接到服务器，请检查网络连接")
		return
	}

	reader := bufio.NewReader(os.Stdin)

	// 检查是否有保存的激活码
	cardNumber := client.settings.GetCardNumber()
	if cardNumber == "" {
		fmt.Print("请输入激活码: ")
		cardNumber, _ = reader.ReadString('\n')
		cardNumber = strings.TrimSpace(cardNumber)

		// 激活新的卡号
		if err := client.activateCard(cardNumber); err != nil {
			fmt.Printf("错误: %v\n", err)
			return
		}
	} else {
		// 如果有保存的激活码，显示主菜单
		fmt.Println("\nCursor Auth 系统")
		fmt.Println("1. 增加使用次数")
		fmt.Println("2. 退出")
		fmt.Print("请选择操作 (1-2): ")
	}

	// 主菜单循环
	for {
		choice, _ := reader.ReadString('\n')
		choice = strings.TrimSpace(choice)

		switch choice {
		case "1":
			if err := client.addCursorTimes(); err != nil {
				fmt.Printf("错误: %v\n", err)
				// 如果认证已失效或卡密过期，要求重新激活
				if err.Error() == "认证已失效，请重新激活" {
					fmt.Print("请输入新的激活码: ")
					cardNumber, _ = reader.ReadString('\n')
					cardNumber = strings.TrimSpace(cardNumber)

					if err := client.activateCard(cardNumber); err != nil {
						fmt.Printf("错误: %v\n", err)
						return
					}
					continue
				}
			}
			fmt.Println("\nCursor Auth 系统")
			fmt.Println("1. 增加使用次数")
			fmt.Println("2. 退出")
			fmt.Print("请选择操作 (1-2): ")

		case "2":
			fmt.Println("感谢使用！")
			return

		default:
			fmt.Println("无效的选择，请重试")
			fmt.Println("\nCursor Auth 系统")
			fmt.Println("1. 增加使用次数")
			fmt.Println("2. 退出")
			fmt.Print("请选择操作 (1-2): ")
		}
	}
}
