const { execSync } = require('child_process');
const fs = require('fs');
const path = require('path');
const os = require('os');

function ensureDirectory(dir) {
  if (!fs.existsSync(dir)) {
    try {
      fs.mkdirSync(dir, { recursive: true });
      console.log(`创建目录成功: ${dir}`);
    } catch (error) {
      console.error(`创建目录失败 ${dir}:`, error);
      process.exit(1);
    }
  }
}

function copyFile(source, target) {
  try {
    fs.copyFileSync(source, target);
    console.log(`复制文件成功: ${source} -> ${target}`);
  } catch (error) {
    console.error(`复制文件失败 ${source}:`, error);
    process.exit(1);
  }
}

// 获取全局安装路径
const globalInstallPath = execSync('npm root -g').toString().trim();
console.log('全局安装路径:', globalInstallPath);

// 确保bin目录存在
const binDir = path.join(__dirname, '..', 'bin');
ensureDirectory(binDir);

// 确保cursor_auth目录存在
const cursorAuthDir = path.join(binDir, 'cursor_auth');
ensureDirectory(cursorAuthDir);

// 检查系统是否安装了Go
try {
  const goVersion = execSync('go version').toString();
  console.log('Go版本:', goVersion);
} catch (error) {
  console.error('请先安装Go环境：https://golang.org/doc/install');
  process.exit(1);
}

// 复制必要的文件到bin目录
const filesToCopy = [
  'go.mod',
  'go.sum',
  '.env',
  'Icon.png'
];

filesToCopy.forEach(file => {
  const sourcePath = path.join(__dirname, '..', file);
  const targetPath = path.join(binDir, file);
  if (fs.existsSync(sourcePath)) {
    copyFile(sourcePath, targetPath);
  } else {
    console.log(`文件不存在，跳过: ${sourcePath}`);
  }
});

// 复制cursor_auth目录下的所有.go文件
const sourceAuthDir = path.join(__dirname, '..', 'cursor_auth');
if (fs.existsSync(sourceAuthDir)) {
  const files = fs.readdirSync(sourceAuthDir);
  files.forEach(file => {
    if (file.endsWith('.go')) {
      const sourcePath = path.join(sourceAuthDir, file);
      const targetPath = path.join(cursorAuthDir, file);
      copyFile(sourcePath, targetPath);
    }
  });
} else {
  console.error('cursor_auth目录不存在:', sourceAuthDir);
  process.exit(1);
}

// 复制main.go
const sourceMainGo = path.join(__dirname, '..', 'main.go');
const targetMainGo = path.join(binDir, 'main.go');
copyFile(sourceMainGo, targetMainGo);

// 编译Go程序
try {
  const platform = os.platform();
  const arch = os.arch();
  const extension = platform === 'win32' ? '.exe' : '';
  const outputPath = path.join(binDir, `cursor-s${extension}`);
  
  console.log('正在编译Go程序...');
  console.log('编译目录:', binDir);
  console.log('输出路径:', outputPath);
  
  // 显示go.mod内容
  console.log('go.mod内容:');
  console.log(fs.readFileSync(path.join(binDir, 'go.mod'), 'utf8'));
  
  // 设置编译环境变量
  const buildEnv = {
    ...process.env,
    CGO_ENABLED: '1',
    GOOS: platform,
    GOARCH: arch,
    GO111MODULE: 'on'
  };

  console.log('编译环境变量:', buildEnv);
  
  execSync('go mod tidy', {
    cwd: binDir,
    stdio: 'inherit',
    env: buildEnv
  });
  
  execSync('go build -v -o ' + outputPath + ' main.go', {
    cwd: binDir,
    stdio: 'inherit',
    env: buildEnv
  });
  
  // 设置可执行权限
  if (platform !== 'win32') {
    fs.chmodSync(outputPath, 0o755);
    console.log(`设置可执行权限成功: ${outputPath}`);
    
    // 同时设置启动脚本的权限
    const startScript = path.join(binDir, 'cursor-s');
    if (fs.existsSync(startScript)) {
      fs.chmodSync(startScript, 0o755);
      console.log(`设置启动脚本权限成功: ${startScript}`);
    }
  }
  
  // 验证编译结果
  if (fs.existsSync(outputPath)) {
    const stats = fs.statSync(outputPath);
    console.log('编译后的文件大小:', stats.size, '字节');
    console.log('文件权限:', stats.mode.toString(8));
  } else {
    throw new Error('编译成功但找不到输出文件');
  }
  
  console.log('安装完成！');
  console.log('二进制文件路径:', outputPath);
  console.log('请尝试运行: cursor-s');
} catch (error) {
  console.error('编译失败：', error);
  process.exit(1);
} 